@@ -85,6 +85,8 @@ typedef struct {
 typedef enum {
 	/* Add item for inter-page changes here */
 	INTER_PAGE_CHANGE_WIFI_MODE = 1,
+	INTER_PAGE_CHANGE_MACSEC_MODE = 2,
+	INTER_PAGE_CHANGE_802_1X_ENABLE = 3,
 } InterPageChangeType;
 
 GType               nm_connection_editor_get_type (void);
