@@ -3,6 +3,7 @@
  *
  * Rodrigo Moya <rodrigo@gnome-db.org>
  * Dan Williams <dcbw@redhat.com>
+ * Lubomir Rintel <lkundrak@v3.sk>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +19,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * Copyright 2007 - 2014 Red Hat, Inc.
+ * Copyright 2007 - 2017 Red Hat, Inc.
  */
 
 #include "nm-default.h"
@@ -31,13 +32,11 @@
 #include "ce-page.h"
 #include "nm-connection-editor.h"
 #include "nm-connection-list.h"
-#include "ce-polkit-button.h"
+#include "ce-polkit.h"
 #include "connection-helpers.h"
 
 extern gboolean nm_ce_keep_above;
 
-G_DEFINE_TYPE (NMConnectionList, nm_connection_list, G_TYPE_OBJECT)
-
 enum {
 	LIST_DONE,
 	EDITING_DONE,
@@ -46,6 +45,30 @@ enum {
 
 static guint list_signals[LIST_LAST_SIGNAL] = { 0 };
 
+struct _NMConnectionListPrivate {
+	GtkWidget *connection_add;
+	GtkWidget *connection_del;
+	GtkWidget *connection_edit;
+	GtkTreeView *connection_list;
+	GtkSearchBar *search_bar;
+	GtkEntry *search_entry;
+	GtkTreeModel *model;
+	GtkTreeModelFilter *filter;
+	GtkTreeSortable *sortable;
+	GType displayed_type;
+
+	NMClient *client;
+
+	gboolean populated;
+};
+
+#define NM_CONNECTION_LIST_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), \
+                                           NM_TYPE_CONNECTION_LIST, \
+                                           NMConnectionListPrivate))
+
+G_DEFINE_TYPE_WITH_CODE (NMConnectionList, nm_connection_list, GTK_TYPE_WINDOW,
+                         G_ADD_PRIVATE (NMConnectionList))
+
 #define COL_ID         0
 #define COL_LAST_USED  1
 #define COL_TIMESTAMP  2
@@ -91,18 +114,19 @@ get_iter_for_connection (NMConnectionList *list,
                          GtkTreeIter *iter)
 {
 	GtkTreeIter types_iter;
+	NMConnectionListPrivate *priv = NM_CONNECTION_LIST_GET_PRIVATE (list);
 
-	if (!gtk_tree_model_get_iter_first (list->model, &types_iter))
+	if (!gtk_tree_model_get_iter_first (priv->model, &types_iter))
 		return FALSE;
 
 	do {
-		if (!gtk_tree_model_iter_children (list->model, iter, &types_iter))
+		if (!gtk_tree_model_iter_children (priv->model, iter, &types_iter))
 			continue;
 
 		do {
 			NMRemoteConnection *candidate = NULL;
 
-			gtk_tree_model_get (list->model, iter,
+			gtk_tree_model_get (priv->model, iter,
 			                    COL_CONNECTION, &candidate,
 			                    -1);
 			if (candidate == connection) {
@@ -110,8 +134,8 @@ get_iter_for_connection (NMConnectionList *list,
 				return TRUE;
 			}
 			g_object_unref (candidate);
-		} while (gtk_tree_model_iter_next (list->model, iter));
-	} while (gtk_tree_model_iter_next (list->model, &types_iter));
+		} while (gtk_tree_model_iter_next (priv->model, iter));
+	} while (gtk_tree_model_iter_next (priv->model, &types_iter));
 
 	return FALSE;
 }
@@ -193,6 +217,7 @@ update_connection_row (NMConnectionList *self,
                        GtkTreeIter *iter,
                        NMRemoteConnection *connection)
 {
+	NMConnectionListPrivate *priv = NM_CONNECTION_LIST_GET_PRIVATE (self);
 	NMSettingConnection *s_con;
 	char *last_used, *id;
 
@@ -201,7 +226,7 @@ update_connection_row (NMConnectionList *self,
 
 	last_used = format_last_used (nm_setting_connection_get_timestamp (s_con));
 	id = g_markup_escape_text (nm_setting_connection_get_id (s_con), -1);
-	gtk_tree_store_set (GTK_TREE_STORE (self->model), iter,
+	gtk_tree_store_set (GTK_TREE_STORE (priv->model), iter,
 	                    COL_ID, id,
 	                    COL_LAST_USED, last_used,
 	                    COL_TIMESTAMP, nm_setting_connection_get_timestamp (s_con),
@@ -210,23 +235,24 @@ update_connection_row (NMConnectionList *self,
 	g_free (last_used);
 	g_free (id);
 
-	gtk_tree_model_filter_refilter (self->filter);
+	gtk_tree_model_filter_refilter (priv->filter);
 }
 
 static void
 delete_slaves_of_connection (NMConnectionList *list, NMConnection *connection)
 {
+	NMConnectionListPrivate *priv = NM_CONNECTION_LIST_GET_PRIVATE (list);
 	const char *uuid, *iface;
 	GtkTreeIter iter, types_iter;
 
-	if (!gtk_tree_model_get_iter_first (list->model, &types_iter))
+	if (!gtk_tree_model_get_iter_first (priv->model, &types_iter))
 		return;
 
 	uuid = nm_connection_get_uuid (connection);
 	iface = nm_connection_get_interface_name (connection);
 
 	do {
-		if (!gtk_tree_model_iter_children (list->model, &iter, &types_iter))
+		if (!gtk_tree_model_iter_children (priv->model, &iter, &types_iter))
 			continue;
 
 		do {
@@ -234,7 +260,7 @@ delete_slaves_of_connection (NMConnectionList *list, NMConnection *connection)
 			NMSettingConnection *s_con;
 			const char *master;
 
-			gtk_tree_model_get (list->model, &iter,
+			gtk_tree_model_get (priv->model, &iter,
 			                    COL_CONNECTION, &candidate,
 			                    -1);
 			s_con = nm_connection_get_setting_connection (NM_CONNECTION (candidate));
@@ -245,8 +271,8 @@ delete_slaves_of_connection (NMConnectionList *list, NMConnection *connection)
 			}
 
 			g_object_unref (candidate);
-		} while (gtk_tree_model_iter_next (list->model, &iter));
-	} while (gtk_tree_model_iter_next (list->model, &types_iter));
+		} while (gtk_tree_model_iter_next (priv->model, &iter));
+	} while (gtk_tree_model_iter_next (priv->model, &types_iter));
 }
 
 
@@ -271,6 +297,7 @@ really_add_connection (FUNC_TAG_NEW_CONNECTION_RESULT_IMPL,
                        gpointer user_data)
 {
 	NMConnectionList *list = user_data;
+	NMConnectionListPrivate *priv = NM_CONNECTION_LIST_GET_PRIVATE (list);
 	NMConnectionEditor *editor;
 
 	if (!connection) {
@@ -285,7 +312,7 @@ really_add_connection (FUNC_TAG_NEW_CONNECTION_RESULT_IMPL,
 	if (connection_supports_ip6 (connection) && !nm_connection_get_setting_ip6_config (connection))
 		nm_connection_add_setting (connection, nm_setting_ip6_config_new ());
 
-	editor = nm_connection_editor_new (GTK_WINDOW (list->dialog), connection, list->client);
+	editor = nm_connection_editor_new (GTK_WINDOW (list), connection, priv->client);
 	if (!editor) {
 		g_signal_emit (list, list_signals[EDITING_DONE], 0, 0);
 		return;
@@ -304,10 +331,13 @@ add_clicked (GtkButton *button, gpointer user_data)
 void
 nm_connection_list_add (NMConnectionList *list)
 {
-	g_return_if_fail (NM_IS_CONNECTION_LIST (list));
+	NMConnectionListPrivate *priv;
 
-	new_connection_dialog (GTK_WINDOW (list->dialog),
-	                       list->client,
+	g_return_if_fail (NM_IS_CONNECTION_LIST (list));
+	priv = NM_CONNECTION_LIST_GET_PRIVATE (list);
+
+	new_connection_dialog (GTK_WINDOW (list),
+	                       priv->client,
 	                       NULL,
 	                       really_add_connection,
 	                       list);
@@ -333,6 +363,7 @@ edit_done_cb (NMConnectionEditor *editor, GtkResponseType response, gpointer use
 static void
 edit_connection (NMConnectionList *list, NMConnection *connection)
 {
+	NMConnectionListPrivate *priv = NM_CONNECTION_LIST_GET_PRIVATE (list);
 	NMConnectionEditor *editor;
 
 	g_return_if_fail (connection != NULL);
@@ -344,9 +375,9 @@ edit_connection (NMConnectionList *list, NMConnection *connection)
 		return;
 	}
 
-	editor = nm_connection_editor_new (GTK_WINDOW (list->dialog),
+	editor = nm_connection_editor_new (GTK_WINDOW (list),
 	                                   NM_CONNECTION (connection),
-	                                   list->client);
+	                                   priv->client);
 	if (editor) {
 		g_signal_connect (editor, "done", G_CALLBACK (edit_done_cb), list);
 		nm_connection_editor_run (editor);
@@ -356,7 +387,10 @@ edit_connection (NMConnectionList *list, NMConnection *connection)
 static void
 do_edit (NMConnectionList *list)
 {
-	edit_connection (list, NM_CONNECTION (get_active_connection (list->connection_list)));
+	NMConnectionListPrivate *priv = NM_CONNECTION_LIST_GET_PRIVATE (list);
+
+	if (gtk_widget_get_sensitive (priv->connection_edit))
+		edit_connection (list, NM_CONNECTION (get_active_connection (priv->connection_list)));
 }
 
 static void
@@ -375,74 +409,112 @@ static void
 delete_clicked (GtkButton *button, gpointer user_data)
 {
 	NMConnectionList *list = user_data;
+	NMConnectionListPrivate *priv = NM_CONNECTION_LIST_GET_PRIVATE (list);
 	NMRemoteConnection *connection;
 
-	connection = get_active_connection (list->connection_list);
+	connection = get_active_connection (priv->connection_list);
 	g_return_if_fail (connection != NULL);
 
-	delete_connection (GTK_WINDOW (list->dialog), connection,
+	delete_connection (GTK_WINDOW (list), connection,
 	                   delete_connection_cb, list);
 }
 
 static void
-pk_button_selection_changed_cb (GtkTreeSelection *selection, gpointer user_data)
+selection_changed_cb (GtkTreeSelection *selection, gpointer user_data)
 {
-	CEPolkitButton *button = user_data;
-	NMConnectionList *list = g_object_get_data (G_OBJECT (button), "NMConnectionList");
+	NMConnectionList *list = user_data;
+	NMConnectionListPrivate *priv = NM_CONNECTION_LIST_GET_PRIVATE (list);
 	GtkTreeIter iter;
 	GtkTreeModel *model;
-	NMRemoteConnection *connection;
+	NMRemoteConnection *connection = NULL;
 	NMSettingConnection *s_con;
 	gboolean sensitive = FALSE;
 
-	if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
-		connection = get_active_connection (list->connection_list);
-		if (connection) {
-			s_con = nm_connection_get_setting_connection (NM_CONNECTION (connection));
-			g_assert (s_con);
+	if (gtk_tree_selection_get_selected (selection, &model, &iter))
+		connection = get_active_connection (priv->connection_list);
 
-			sensitive = !nm_setting_connection_get_read_only (s_con);
-		}
+	if (connection) {
+		s_con = nm_connection_get_setting_connection (NM_CONNECTION (connection));
+		g_assert (s_con);
+
+		sensitive = !nm_setting_connection_get_read_only (s_con);
+
+		ce_polkit_set_widget_validation_error (priv->connection_edit,
+		                                       sensitive ? NULL : _("Connection cannot be modified"));
+		ce_polkit_set_widget_validation_error (priv->connection_del,
+		                                       sensitive ? NULL : _("Connection cannot be deleted"));
+	} else {
+		ce_polkit_set_widget_validation_error (priv->connection_edit,
+		                                       _("Select a connection to edit"));
+		ce_polkit_set_widget_validation_error (priv->connection_del,
+		                                       _("Select a connection to delete"));
+	}
+}
+
+static void
+list_close_cb (GtkDialog *dialog, gpointer user_data)
+{
+	g_signal_emit (NM_CONNECTION_LIST (user_data),
+	               list_signals[LIST_DONE],
+	               0,
+	               GTK_RESPONSE_CLOSE);
+}
+
+static gboolean
+key_press_cb (GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+	NMConnectionList *list = user_data;
+	NMConnectionListPrivate *priv = NM_CONNECTION_LIST_GET_PRIVATE (list);
+	GdkEventKey *key_event = (GdkEventKey *) event;
+
+	if (gtk_search_bar_handle_event (priv->search_bar, event) == GDK_EVENT_STOP)
+		return GDK_EVENT_STOP;
+
+	if (key_event->keyval == GDK_KEY_Escape) {
+		if (gtk_search_bar_get_search_mode (priv->search_bar))
+			gtk_search_bar_set_search_mode (priv->search_bar, FALSE);
+		else
+			gtk_window_close (GTK_WINDOW (user_data));
+		return GDK_EVENT_STOP;
 	}
 
-	ce_polkit_button_set_validation_error (button, sensitive ? NULL : _("Connection cannot be modified"));
+	return GDK_EVENT_PROPAGATE;
+}
+
+static gboolean
+start_search (GtkTreeView *treeview, gpointer user_data)
+{
+	NMConnectionList *list = user_data;
+	NMConnectionListPrivate *priv = NM_CONNECTION_LIST_GET_PRIVATE (list);
+
+	gtk_search_bar_set_search_mode (priv->search_bar, TRUE);
+
+	return TRUE;
 }
 
 static void
-connection_double_clicked_cb (GtkTreeView *tree_view,
-                              GtkTreePath *path,
-                              GtkTreeViewColumn *column,
-                              gpointer user_data)
+search_changed (GtkSearchEntry *entry, gpointer user_data)
 {
-	GtkButton *button = user_data;
+	NMConnectionList *list = user_data;
+	NMConnectionListPrivate *priv = NM_CONNECTION_LIST_GET_PRIVATE (list);
 
-	if (ce_polkit_button_get_actionable (CE_POLKIT_BUTTON (button)))
-		gtk_button_clicked (button);
-}
-
-static void
-dialog_response_cb (GtkDialog *dialog, guint response, gpointer user_data)
-{
-	gtk_widget_hide (GTK_WIDGET (dialog));
+	gtk_tree_model_filter_refilter (priv->filter);
+	gtk_tree_view_expand_all (priv->connection_list);
 }
 
 static void
 nm_connection_list_init (NMConnectionList *list)
 {
+	gtk_widget_init_template (GTK_WIDGET (list));
 }
 
 static void
 dispose (GObject *object)
 {
 	NMConnectionList *list = NM_CONNECTION_LIST (object);
+	NMConnectionListPrivate *priv = NM_CONNECTION_LIST_GET_PRIVATE (list);
 
-	if (list->dialog)
-		gtk_widget_hide (list->dialog);
-
-	if (list->gui)
-		g_object_unref (list->gui);
-	if (list->client)
-		g_object_unref (list->client);
+	g_clear_object (&priv->client);
 
 	G_OBJECT_CLASS (nm_connection_list_parent_class)->dispose (object);
 }
@@ -451,6 +523,7 @@ static void
 nm_connection_list_class_init (NMConnectionListClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
 	/* virtual methods */
 	object_class->dispose = dispose;
@@ -471,6 +544,26 @@ nm_connection_list_class_init (NMConnectionListClass *klass)
 		              G_STRUCT_OFFSET (NMConnectionListClass, done),
 		              NULL, NULL, NULL,
 		              G_TYPE_NONE, 1, G_TYPE_INT);
+
+	/* Initialize the widget template */
+        gtk_widget_class_set_template_from_resource (widget_class,
+	                                             "/org/freedesktop/network-manager-applet/nm-connection-list.ui");
+
+        gtk_widget_class_bind_template_child_private (widget_class, NMConnectionList, connection_list);
+        gtk_widget_class_bind_template_child_private (widget_class, NMConnectionList, connection_add);
+        gtk_widget_class_bind_template_child_private (widget_class, NMConnectionList, connection_del);
+        gtk_widget_class_bind_template_child_private (widget_class, NMConnectionList, connection_edit);
+        gtk_widget_class_bind_template_child_private (widget_class, NMConnectionList, search_bar);
+        gtk_widget_class_bind_template_child_private (widget_class, NMConnectionList, search_entry);
+
+        gtk_widget_class_bind_template_callback (widget_class, add_clicked);
+        gtk_widget_class_bind_template_callback (widget_class, do_edit);
+        gtk_widget_class_bind_template_callback (widget_class, delete_clicked);
+        gtk_widget_class_bind_template_callback (widget_class, list_close_cb);
+        gtk_widget_class_bind_template_callback (widget_class, selection_changed_cb);
+        gtk_widget_class_bind_template_callback (widget_class, key_press_cb);
+        gtk_widget_class_bind_template_callback (widget_class, start_search);
+        gtk_widget_class_bind_template_callback (widget_class, search_changed);
 }
 
 static void
@@ -548,23 +641,59 @@ timestamp_sort_func (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpoint
 	return time_b - time_a;
 }
 
+static gboolean
+has_visible_children (NMConnectionList *self, GtkTreeModel *model, GtkTreeIter *parent)
+{
+	NMConnectionListPrivate *priv = NM_CONNECTION_LIST_GET_PRIVATE (self);
+	GtkTreeIter iter;
+	const char *search;
+	char *id = NULL;
+
+	if (!gtk_search_bar_get_search_mode (priv->search_bar))
+		return gtk_tree_model_iter_has_child  (model, parent);
+
+	if (!gtk_tree_model_iter_children (model, &iter, parent))
+		return FALSE;
+
+	search = gtk_entry_get_text (GTK_ENTRY (priv->search_entry));
+	do {
+		gtk_tree_model_get (model, &iter, COL_ID, &id, -1);
+		if (strcasestr (id, search) != NULL) {
+			g_free (id);
+			return TRUE;
+		}
+		g_free (id);
+	} while (gtk_tree_model_iter_next (model, &iter));
+
+	return FALSE;
+}
+
 static gboolean
 tree_model_visible_func (GtkTreeModel *model,
                          GtkTreeIter *iter,
                          gpointer user_data)
 {
 	NMConnectionList *self = user_data;
+	NMConnectionListPrivate *priv = NM_CONNECTION_LIST_GET_PRIVATE (self);
 	gs_unref_object NMConnection *connection = NULL;
 	NMSettingConnection *s_con;
 	const char *master;
 	const char *slave_type;
+	gs_free char *id = NULL;
 
-	gtk_tree_model_get (model, iter, COL_CONNECTION, &connection, -1);
+	gtk_tree_model_get (model, iter,
+	                    COL_ID, &id,
+	                    COL_CONNECTION, &connection,
+	                    -1);
 	if (!connection) {
-		/* Top-level type nodes are visible iff they have children */
-		return gtk_tree_model_iter_has_child  (model, iter);
+		/* Top-level type nodes are visible iff they have visible children */
+		return has_visible_children (self, model, iter);
 	}
 
+	if (   gtk_search_bar_get_search_mode (priv->search_bar)
+	    && strcasestr (id, gtk_entry_get_text (GTK_ENTRY (priv->search_entry))) == NULL)
+		return FALSE;
+
 	/* A connection node is visible unless it is a slave to a known
 	 * bond or team or bridge.
 	 */
@@ -582,7 +711,7 @@ tree_model_visible_func (GtkTreeModel *model,
 	    && g_strcmp0 (slave_type, NM_SETTING_BRIDGE_SETTING_NAME) != 0)
 		return TRUE;
 
-	if (nm_client_get_connection_by_uuid (self->client, master))
+	if (nm_client_get_connection_by_uuid (priv->client, master))
 		return FALSE;
 	if (nm_connection_editor_get_master (connection))
 		return FALSE;
@@ -592,9 +721,28 @@ tree_model_visible_func (GtkTreeModel *model,
 	return TRUE;
 }
 
+static gboolean
+connection_list_equal (GtkTreeModel *model, gint column, const gchar *key,
+                       GtkTreeIter *iter, gpointer user_data)
+{
+	gs_free char *id = NULL;
+	gs_unref_object NMConnection *connection = NULL;
+
+	gtk_tree_model_get (model, iter,
+	                    COL_ID, &id,
+	                    COL_CONNECTION, &connection,
+	                    -1);
+
+	if (!connection)
+		return TRUE;
+
+	return strcasestr (id, key) == NULL;
+}
+
 static void
 initialize_treeview (NMConnectionList *self)
 {
+	NMConnectionListPrivate *priv = NM_CONNECTION_LIST_GET_PRIVATE (self);
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
 	GtkTreeSelection *selection;
@@ -604,7 +752,7 @@ initialize_treeview (NMConnectionList *self)
 	int i;
 
 	/* Model */
-	self->model = GTK_TREE_MODEL (gtk_tree_store_new (8, G_TYPE_STRING,
+	priv->model = GTK_TREE_MODEL (gtk_tree_store_new (8, G_TYPE_STRING,
 	                                                     G_TYPE_STRING,
 	                                                     G_TYPE_UINT64,
 	                                                     G_TYPE_OBJECT,
@@ -614,21 +762,23 @@ initialize_treeview (NMConnectionList *self)
 	                                                     G_TYPE_INT));
 
 	/* Filter */
-	self->filter = GTK_TREE_MODEL_FILTER (gtk_tree_model_filter_new (self->model, NULL));
-	gtk_tree_model_filter_set_visible_func (self->filter,
+	priv->filter = GTK_TREE_MODEL_FILTER (gtk_tree_model_filter_new (priv->model, NULL));
+	gtk_tree_model_filter_set_visible_func (priv->filter,
 	                                        tree_model_visible_func,
 	                                        self, NULL);
 
 	/* Sortable */
-	self->sortable = GTK_TREE_SORTABLE (gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (self->filter)));
-	gtk_tree_sortable_set_default_sort_func (self->sortable, NULL, NULL, NULL);
-	gtk_tree_sortable_set_sort_func (self->sortable, COL_TIMESTAMP, timestamp_sort_func,
-	                                 self->sortable, NULL);
-	gtk_tree_sortable_set_sort_func (self->sortable, COL_ID, id_sort_func,
-	                                 self->sortable, NULL);
-	gtk_tree_sortable_set_sort_column_id (self->sortable, COL_TIMESTAMP, GTK_SORT_ASCENDING);
+	priv->sortable = GTK_TREE_SORTABLE (gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (priv->filter)));
+	gtk_tree_sortable_set_default_sort_func (priv->sortable, NULL, NULL, NULL);
+	gtk_tree_sortable_set_sort_func (priv->sortable, COL_TIMESTAMP, timestamp_sort_func,
+	                                 priv->sortable, NULL);
+	gtk_tree_sortable_set_sort_func (priv->sortable, COL_ID, id_sort_func,
+	                                 priv->sortable, NULL);
+	gtk_tree_sortable_set_sort_column_id (priv->sortable, COL_TIMESTAMP, GTK_SORT_ASCENDING);
 
-	gtk_tree_view_set_model (self->connection_list, GTK_TREE_MODEL (self->sortable));
+	gtk_tree_view_set_model (priv->connection_list, GTK_TREE_MODEL (priv->sortable));
+	gtk_tree_view_set_search_equal_func (priv->connection_list, connection_list_equal, NULL, NULL);
+	gtk_tree_view_set_search_entry (priv->connection_list, priv->search_entry);
 
 	/* Name column */
 	renderer = gtk_cell_renderer_text_new ();
@@ -639,7 +789,7 @@ initialize_treeview (NMConnectionList *self)
 	gtk_tree_view_column_set_expand (column, TRUE);
 	gtk_tree_view_column_set_sort_column_id (column, COL_ID);
 	g_signal_connect (column, "clicked", G_CALLBACK (column_header_clicked_cb), GINT_TO_POINTER (COL_ID));
-	gtk_tree_view_append_column (self->connection_list, column);
+	gtk_tree_view_append_column (priv->connection_list, column);
 
 	/* Last Used column */
 	renderer = g_object_new (GTK_TYPE_CELL_RENDERER_TEXT,
@@ -651,10 +801,10 @@ initialize_treeview (NMConnectionList *self)
 	                                                   NULL);
 	gtk_tree_view_column_set_sort_column_id (column, COL_TIMESTAMP);
 	g_signal_connect (column, "clicked", G_CALLBACK (column_header_clicked_cb), GINT_TO_POINTER (COL_TIMESTAMP));
-	gtk_tree_view_append_column (self->connection_list, column);
+	gtk_tree_view_append_column (priv->connection_list, column);
 
 	/* Selection */
-	selection = gtk_tree_view_get_selection (self->connection_list);
+	selection = gtk_tree_view_get_selection (priv->connection_list);
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
 
 	/* Fill in connection types */
@@ -665,8 +815,8 @@ initialize_treeview (NMConnectionList *self)
 		id = g_strdup_printf ("<b>%s</b>", tmp);
 		g_free (tmp);
 
-		gtk_tree_store_append (GTK_TREE_STORE (self->model), &iter, NULL);
-		gtk_tree_store_set (GTK_TREE_STORE (self->model), &iter,
+		gtk_tree_store_append (GTK_TREE_STORE (priv->model), &iter, NULL);
+		gtk_tree_store_set (GTK_TREE_STORE (priv->model), &iter,
 		                    COL_ID, id,
 		                    COL_GTYPE0, types[i].setting_types[0],
 		                    COL_GTYPE1, types[i].setting_types[1],
@@ -680,49 +830,19 @@ initialize_treeview (NMConnectionList *self)
 static void
 add_connection_buttons (NMConnectionList *self)
 {
-	GtkWidget *button, *box;
-	GtkTreeSelection *selection;
+	NMConnectionListPrivate *priv = NM_CONNECTION_LIST_GET_PRIVATE (self);
 
-	selection = gtk_tree_view_get_selection (self->connection_list);
+	ce_polkit_connect_widget (priv->connection_edit,
+	                          _("Edit the selected connection"),
+	                          _("Authenticate to edit the selected connection"),
+	                          priv->client,
+	                          NM_CLIENT_PERMISSION_SETTINGS_MODIFY_SYSTEM);
 
-	/* Add */
-	button = GTK_WIDGET (gtk_builder_get_object (self->gui, "connection_add"));
-	g_signal_connect (button, "clicked", G_CALLBACK (add_clicked), self);
-
-	box = GTK_WIDGET (gtk_builder_get_object (self->gui, "connection_button_box"));
-
-	/* Edit */
-	button = ce_polkit_button_new (_("_Edit"),
-	                               _("Edit the selected connection"),
-	                               _("Authenticate to edit the selected connection"),
-	                               GTK_STOCK_EDIT,
-	                               self->client,
-	                               NM_CLIENT_PERMISSION_SETTINGS_MODIFY_SYSTEM);
-	g_object_set_data (G_OBJECT (button), "NMConnectionList", self);
-	gtk_button_set_use_underline (GTK_BUTTON (button), TRUE);
-	gtk_box_pack_end (GTK_BOX (box), button, TRUE, TRUE, 0);
-
-	g_signal_connect_swapped (button, "clicked", G_CALLBACK (do_edit), self);
-	g_signal_connect (self->connection_list, "row-activated", G_CALLBACK (connection_double_clicked_cb), button);
-	g_signal_connect (selection, "changed", G_CALLBACK (pk_button_selection_changed_cb), button);
-	pk_button_selection_changed_cb (selection, button);
-
-	/* Delete */
-	button = ce_polkit_button_new (_("_Delete"),
-	                               _("Delete the selected connection"),
-	                               _("Authenticate to delete the selected connection"),
-	                               GTK_STOCK_DELETE,
-	                               self->client,
-	                               NM_CLIENT_PERMISSION_SETTINGS_MODIFY_SYSTEM);
-	g_object_set_data (G_OBJECT (button), "NMConnectionList", self);
-	gtk_button_set_use_underline (GTK_BUTTON (button), TRUE);
-	gtk_box_pack_end (GTK_BOX (box), button, TRUE, TRUE, 0);
-
-	g_signal_connect (button, "clicked", G_CALLBACK (delete_clicked), self);
-	g_signal_connect (selection, "changed", G_CALLBACK (pk_button_selection_changed_cb), button);
-	pk_button_selection_changed_cb (selection, button);
-
-	gtk_widget_show_all (box);
+	ce_polkit_connect_widget (priv->connection_del,
+	                          _("Delete the selected connection"),
+	                          _("Authenticate to delete the selected connection"),
+	                          priv->client,
+	                          NM_CLIENT_PERMISSION_SETTINGS_MODIFY_SYSTEM);
 }
 
 static void
@@ -731,13 +851,14 @@ connection_removed (NMClient *client,
                     gpointer user_data)
 {
 	NMConnectionList *self = NM_CONNECTION_LIST (user_data);
+	NMConnectionListPrivate *priv = NM_CONNECTION_LIST_GET_PRIVATE (self);
 	GtkTreeIter iter, parent_iter;
 
 	if (get_iter_for_connection (self, connection, &iter)) {
-		gtk_tree_model_iter_parent (self->model, &parent_iter, &iter);
-		gtk_tree_store_remove (GTK_TREE_STORE (self->model), &iter);
+		gtk_tree_model_iter_parent (priv->model, &parent_iter, &iter);
+		gtk_tree_store_remove (GTK_TREE_STORE (priv->model), &iter);
 	}
-	gtk_tree_model_filter_refilter (self->filter);
+	gtk_tree_model_filter_refilter (priv->filter);
 }
 
 static void
@@ -760,6 +881,7 @@ get_parent_iter_for_connection (NMConnectionList *list,
                                 NMRemoteConnection *connection,
                                 GtkTreeIter *iter)
 {
+	NMConnectionListPrivate *priv = NM_CONNECTION_LIST_GET_PRIVATE (list);
 	NMSettingConnection *s_con;
 	const char *str_type;
 	GType type, row_type0, row_type1, row_type2;
@@ -774,16 +896,16 @@ get_parent_iter_for_connection (NMConnectionList *list,
 
 	type = nm_setting_lookup_type (str_type);
 
-	if (gtk_tree_model_get_iter_first (list->model, iter)) {
+	if (gtk_tree_model_get_iter_first (priv->model, iter)) {
 		do {
-			gtk_tree_model_get (list->model, iter,
+			gtk_tree_model_get (priv->model, iter,
 			                    COL_GTYPE0, &row_type0,
 			                    COL_GTYPE1, &row_type1,
 			                    COL_GTYPE2, &row_type2,
 			                    -1);
 			if (row_type0 == type || row_type1 == type || row_type2 == type)
 				return TRUE;
-		} while (gtk_tree_model_iter_next (list->model, iter));
+		} while (gtk_tree_model_iter_next (priv->model, iter));
 	}
 
 	return FALSE;
@@ -795,6 +917,7 @@ connection_added (NMClient *client,
                   gpointer user_data)
 {
 	NMConnectionList *self = NM_CONNECTION_LIST (user_data);
+	NMConnectionListPrivate *priv = NM_CONNECTION_LIST_GET_PRIVATE (self);
 	GtkTreeIter parent_iter, iter;
 	NMSettingConnection *s_con;
 	char *last_used, *id;
@@ -809,8 +932,8 @@ connection_added (NMClient *client,
 
 	id = g_markup_escape_text (nm_setting_connection_get_id (s_con), -1);
 
-	gtk_tree_store_append (GTK_TREE_STORE (self->model), &iter, &parent_iter);
-	gtk_tree_store_set (GTK_TREE_STORE (self->model), &iter,
+	gtk_tree_store_append (GTK_TREE_STORE (priv->model), &iter, &parent_iter);
+	gtk_tree_store_set (GTK_TREE_STORE (priv->model), &iter,
 	                    COL_ID, id,
 	                    COL_LAST_USED, last_used,
 	                    COL_TIMESTAMP, nm_setting_connection_get_timestamp (s_con),
@@ -820,81 +943,66 @@ connection_added (NMClient *client,
 	g_free (id);
 	g_free (last_used);
 
-	if (self->displayed_type) {
+	if (priv->displayed_type) {
 		GType added_type0, added_type1, added_type2;
 
-		gtk_tree_model_get (self->model, &parent_iter,
+		gtk_tree_model_get (priv->model, &parent_iter,
 		                    COL_GTYPE0, &added_type0,
 		                    COL_GTYPE1, &added_type1,
 		                    COL_GTYPE2, &added_type2,
 		                    -1);
-		if (   added_type0 != self->displayed_type
-		    && added_type1 != self->displayed_type
-		    && added_type2 != self->displayed_type)
+		if (   added_type0 != priv->displayed_type
+		    && added_type1 != priv->displayed_type
+		    && added_type2 != priv->displayed_type)
 			expand = FALSE;
 	}
 
 	if (expand) {
 		GtkTreePath *path, *filtered_path;
 
-		path = gtk_tree_model_get_path (self->model, &parent_iter);
-		filtered_path = gtk_tree_model_filter_convert_child_path_to_path (self->filter, path);
-		gtk_tree_view_expand_row (self->connection_list, filtered_path, FALSE);
+		path = gtk_tree_model_get_path (priv->model, &parent_iter);
+		filtered_path = gtk_tree_model_filter_convert_child_path_to_path (priv->filter, path);
+		gtk_tree_view_expand_row (priv->connection_list, filtered_path, FALSE);
 		gtk_tree_path_free (filtered_path);
 		gtk_tree_path_free (path);
 	}
 
 	g_signal_connect (client, NM_CLIENT_CONNECTION_REMOVED, G_CALLBACK (connection_removed), self);
 	g_signal_connect (connection, NM_CONNECTION_CHANGED, G_CALLBACK (connection_changed), self);
-	gtk_tree_model_filter_refilter (self->filter);
+	gtk_tree_model_filter_refilter (priv->filter);
 }
 
 NMConnectionList *
 nm_connection_list_new (void)
 {
 	NMConnectionList *list;
+	NMConnectionListPrivate *priv;
 	GError *error = NULL;
-	const char *objects[] = { "NMConnectionList", NULL };
 
 	list = g_object_new (NM_TYPE_CONNECTION_LIST, NULL);
 	if (!list)
 		return NULL;
 
-	/* load GUI */
-	list->gui = gtk_builder_new ();
-
-	if (!gtk_builder_add_objects_from_resource (list->gui,
-	                                            "/org/freedesktop/network-manager-applet/nm-connection-editor.ui",
-	                                            (char **) objects,
-	                                            &error)) {
-		g_warning ("Couldn't load builder resource: %s", error->message);
-		g_error_free (error);
-		goto error;
-	}
+	priv = NM_CONNECTION_LIST_GET_PRIVATE (list);
 
 	gtk_window_set_default_icon_name ("preferences-system-network");
 
-	list->client = nm_client_new (NULL, &error);
-	if (!list->client) {
+	priv->client = nm_client_new (NULL, &error);
+	if (!priv->client) {
 		g_warning ("Couldn't construct the client instance: %s", error->message);
 		g_error_free (error);
 		goto error;
 	}
-	g_signal_connect (list->client,
+	g_signal_connect (priv->client,
 	                  NM_CLIENT_CONNECTION_ADDED,
 	                  G_CALLBACK (connection_added),
 	                  list);
 
-	list->connection_list = GTK_TREE_VIEW (gtk_builder_get_object (list->gui, "connection_list"));
-	initialize_treeview (list);
 	add_connection_buttons (list);
+	initialize_treeview (list);
 
-	list->dialog = GTK_WIDGET (gtk_builder_get_object (list->gui, "NMConnectionList"));
-	if (!list->dialog)
-		goto error;
 	if (nm_ce_keep_above)
-		gtk_window_set_keep_above (GTK_WINDOW (list->dialog), TRUE);
-	g_signal_connect (G_OBJECT (list->dialog), "response", G_CALLBACK (dialog_response_cb), list);
+		gtk_window_set_keep_above (GTK_WINDOW (list), TRUE);
 
 	return list;
 
@@ -906,21 +1014,26 @@ error:
 void
 nm_connection_list_set_type (NMConnectionList *self, GType ctype)
 {
-	g_return_if_fail (NM_IS_CONNECTION_LIST (self));
+	NMConnectionListPrivate *priv;
 
-	self->displayed_type = ctype;
+	g_return_if_fail (NM_IS_CONNECTION_LIST (self));
+	priv = NM_CONNECTION_LIST_GET_PRIVATE (self);
+
+	priv->displayed_type = ctype;
 }
 
 void
-nm_connection_list_create (NMConnectionList *self,
+nm_connection_list_create (NMConnectionList *list,
                            GType ctype,
                            const char *detail,
                            const char *import_filename)
 {
+	NMConnectionListPrivate *priv;
 	ConnectionTypeData *types;
 	int i;
 
-	g_return_if_fail (NM_IS_CONNECTION_LIST (self));
+	g_return_if_fail (NM_IS_CONNECTION_LIST (list));
+	priv = NM_CONNECTION_LIST_GET_PRIVATE (list);
 
 	types = get_connection_type_list ();
 	for (i = 0; types[i].name; i++) {
@@ -946,25 +1059,27 @@ nm_connection_list_create (NMConnectionList *self,
 			if (!connection)
 				return;
 		}
-		new_connection_of_type (GTK_WINDOW (self->dialog),
+		new_connection_of_type (GTK_WINDOW (list),
 		                        detail,
 		                        NULL,
 		                        connection,
-		                        self->client,
+		                        priv->client,
 		                        types[i].new_connection_func,
 		                        really_add_connection,
-		                        self);
+		                        list);
 	}
 }
 
 void
 nm_connection_list_edit (NMConnectionList *self, const gchar *uuid)
 {
+	NMConnectionListPrivate *priv;
 	NMConnection *connection;
 
 	g_return_if_fail (NM_IS_CONNECTION_LIST (self));
+	priv = NM_CONNECTION_LIST_GET_PRIVATE (self);
 
-	connection = (NMConnection *) nm_client_get_connection_by_uuid (self->client, uuid);
+	connection = (NMConnection *) nm_client_get_connection_by_uuid (priv->client, uuid);
 	if (!connection) {
 		nm_connection_editor_error (NULL,
 		                            _("Error editing connection"),
@@ -975,48 +1090,33 @@ nm_connection_list_edit (NMConnectionList *self, const gchar *uuid)
 	edit_connection (self, connection);
 }
 
-static void
-list_response_cb (GtkDialog *dialog, gint response, gpointer user_data)
-{
-	g_signal_emit (NM_CONNECTION_LIST (user_data), list_signals[LIST_DONE], 0, response);
-}
-
-static void
-list_close_cb (GtkDialog *dialog, gpointer user_data)
-{
-	gtk_dialog_response (dialog, GTK_RESPONSE_CLOSE);
-}
-
 void
 nm_connection_list_present (NMConnectionList *list)
 {
+	NMConnectionListPrivate *priv;
 	const GPtrArray *all_cons;
 	GtkTreePath *path;
 	GtkTreeIter iter;
 	int i;
 
 	g_return_if_fail (NM_IS_CONNECTION_LIST (list));
+	priv = NM_CONNECTION_LIST_GET_PRIVATE (list);
 
-	if (!list->populated) {
+	if (!priv->populated) {
 		/* Fill the treeview initially */
-		all_cons = nm_client_get_connections (list->client);
+		all_cons = nm_client_get_connections (priv->client);
 		for (i = 0; i < all_cons->len; i++)
-			connection_added (list->client, all_cons->pdata[i], list);
-		if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (list->sortable), &iter)) {
-			path = gtk_tree_model_get_path (GTK_TREE_MODEL (list->sortable), &iter);
-			gtk_tree_view_scroll_to_cell (list->connection_list,
+			connection_added (priv->client, all_cons->pdata[i], list);
+		if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->sortable), &iter)) {
+			path = gtk_tree_model_get_path (GTK_TREE_MODEL (priv->sortable), &iter);
+			gtk_tree_view_scroll_to_cell (priv->connection_list,
 			                              path, NULL,
 			                              FALSE, 0, 0);
 			gtk_tree_path_free (path);
 		}
 
-		g_signal_connect (G_OBJECT (list->dialog), "response",
-			              G_CALLBACK (list_response_cb), list);
-		g_signal_connect (G_OBJECT (list->dialog), "close",
-			              G_CALLBACK (list_close_cb), list);
-		list->populated = TRUE;
+		priv->populated = TRUE;
 	}
 
-	gtk_window_present (GTK_WINDOW (list->dialog));
+	gtk_window_present (GTK_WINDOW (list));
 }
-
