@@ -501,12 +501,6 @@ enable_toggled_cb (GtkToggleButton *button, EnableInfo *info)
 	ce_page_changed (info->page);
 }
 
-static void
-free_enable_info (gpointer data, GClosure *closure)
-{
-	g_free (data);
-}
-
 static void
 feature_setup (CEPageDcb *self, NMSettingDcb *s_dcb, const Feature *f)
 {
@@ -527,7 +521,8 @@ feature_setup (CEPageDcb *self, NMSettingDcb *s_dcb, const Feature *f)
 	info = g_malloc0 (sizeof (EnableInfo));
 	info->f = f;
 	info->page = parent;
-	g_signal_connect_data (widget, "toggled", G_CALLBACK (enable_toggled_cb), info, free_enable_info, 0);
+	g_signal_connect (widget, "toggled", G_CALLBACK (enable_toggled_cb), info);
+	g_object_weak_ref (G_OBJECT (widget), (GWeakNotify) g_free, info);
 
 	/* Advertise */
 	widget = get_widget (parent->builder, f->prefix, "_advertise_checkbutton");
