@@ -380,7 +380,7 @@ add_connection (FUNC_TAG_NEW_CONNECTION_RESULT_IMPL,
 	iface_name = gtk_entry_get_text (priv->interface_name);
 	if (!*iface_name)
 		iface_name = nm_connection_get_interface_name (connection);
-	if (!iface_name || !nm_utils_iface_valid_name (iface_name))
+	if (!iface_name || !nm_utils_is_valid_iface_name (iface_name, NULL))
 		iface_name = nm_connection_get_id (connection);
 	name = g_strdup_printf (_("%s slave %d"), iface_name,
 	                        gtk_tree_model_iter_n_children (priv->connections_model, NULL) + 1);
@@ -431,7 +431,9 @@ get_selected_connection (CEPageMaster *self)
 	if (gtk_tree_model_get_iter (model, &iter, (GtkTreePath *) selected_rows->data))
 		gtk_tree_model_get (model, &iter, 0, &connection, -1);
 
-	g_list_free_full (selected_rows, (GDestroyNotify) gtk_tree_path_free);
+	/* free memory */
+	g_list_foreach (selected_rows, (GFunc) gtk_tree_path_free, NULL);
+	g_list_free (selected_rows);
 
 	return connection;
 }
